"""
open/dulcinea/lib/archive.py
"""
from qp.lib.stored_file import new_file
from durus.utils import BytesIO
from os.path import basename
from qp.fill.static import FileStream
from qp.pub.common import not_found, get_user, get_hit
import mimetypes
import tarfile
import zipfile
import struct

class ArchiveWrapper (object):

    def __init__(self, path_to_archive):
        self.path_to_archive = path_to_archive
        self.archive = None

    def get_archive(self):
        if not self.archive:
            self.archive = self.open_archive(self.path_to_archive)
        return self.archive

    def open_archive(self, path_to_archive):
        """(path_to_archive : str)

        Return a reference to archive reading class saved on self.archive
        """
        raise NotImplementedError

    def get_names(self):
        """() -> [str]

        Return a list of file and directory names in the order they appear in
        the archive.  Directory names end with a slash '/' character.
        """
        raise NotImplementedError

    def has_member(self, member_name):
        """(name : str) -> bool

        True if member is in the archive
        """
        raise NotImplementedError

    def get_member_file(self, member_name):
        """(member_name : str) -> file
        """
        raise NotImplementedError

    def get_member_response(self, member_name):
        """(member_name : str) -> str | Stream

        Return the member named by member_name as a string or if member is
        streamable as a Stream
        """
        raise NotImplementedError

    def get_member(self, member_name):
        """(member_name : str) -> str

        Return the member named by member_name as a string.
        """
        raise NotImplementedError

    def get_member_stored_file(self, member_name):
        """(member_name : str) -> StoredFile | None

        Return the member named by member_name as a StoredFile
        """
        if not self.has_member(member_name):
            return None
        stored_file = new_file(self.get_member_file(member_name))
        filename = basename(member_name)
        stored_file.set_mime_type(mimetypes.guess_type(filename.lower())[0])
        stored_file.set_filename(filename)
        if get_hit():
            stored_file.set_owner(get_user())
        return stored_file

class TarFileWrapper (ArchiveWrapper):

    def open_archive(self, path_to_archive):
        try:
            return tarfile.open(path_to_archive, 'r:gz')
        except tarfile.ReadError:
            try:
                return tarfile.open(path_to_archive, 'r:')
            except:
                not_found("Can't open tar file")

    def get_names(self):
        fixed_names = []
        for tarinfo in self.get_archive().getmembers():
            name = tarinfo.name
            if name.endswith('//'):
                fixed_names.append(name[:-1])
            else:
                if not name.endswith('/') and tarinfo.isdir():
                    name += '/'
                fixed_names.append(name)
        return fixed_names

    def has_member(self, member_name):
        if member_name.endswith('/'):
            try:
                member = self.get_archive().getmember(member_name)
            except (KeyError, NameError):
                member = None
            if member is None:
                try:
                    member = self.get_archive().getmember(member_name[:-1])
                except (KeyError, NameError):
                    member = None
            if member is None:
                try:
                    member = self.get_archive().getmember(member_name + '/')
                except (KeyError, NameError):
                    member = None
            return member and member.isdir()
        else:
            try:
                member = self.get_archive().getmember(member_name)
                return member.isfile()
            except (KeyError, NameError):
                return False

    def get_member_file(self, member_name):
        try:
            return self.get_archive().extractfile(member_name)
        except KeyError:
            return None
        return None

    def get_member_response(self, member_name):
        return FileStream(self.get_member_file(member_name))

    def get_member(self, member_name):
        return self.get_member_file(member_name).read()

class NearZipFile (zipfile.ZipFile):
    """
    This is identical to a standard ZipFile, except that does not fail
    when opening archives that have "extra" fields of length 1, 2, or 3.
    """
    def _RealGetContents(self):
        """Read in the table of contents for the ZIP file."""
        fp = self.fp
        endrec = zipfile._EndRecData(fp)
        if not endrec:
            raise zipfile.BadZipfile("File is not a zip file")
        if self.debug > 1:
            print(endrec)
        size_cd = endrec[zipfile._ECD_SIZE]             # bytes in central directory
        offset_cd = endrec[zipfile._ECD_OFFSET]         # offset of central directory
        self.comment = endrec[zipfile._ECD_COMMENT]     # archive comment

        # "concat" is zero, unless zip was concatenated to another file
        concat = endrec[zipfile._ECD_LOCATION] - size_cd - offset_cd
        if endrec[zipfile._ECD_SIGNATURE] == zipfile.stringEndArchive64:
            # If Zip64 extension structures are present, account for them
            concat -= (zipfile.sizeEndCentDir64 + zipfile.sizeEndCentDir64Locator)

        if self.debug > 2:
            inferred = concat + offset_cd
            print("given, inferred, offset %s %s %s" % (offset_cd, inferred, concat))
        # self.start_dir:  Position of start of central directory
        self.start_dir = offset_cd + concat
        fp.seek(self.start_dir, 0)
        data = fp.read(size_cd)
        fp = BytesIO(data)
        total = 0
        while total < size_cd:
            centdir = fp.read(zipfile.sizeCentralDir)
            if centdir[0:4] != zipfile.stringCentralDir:
                raise zipfile.BadZipfile("Bad magic number for central directory")
            centdir = struct.unpack(zipfile.structCentralDir, centdir)
            if self.debug > 2:
                print(centdir)
            filename = fp.read(centdir[zipfile._CD_FILENAME_LENGTH])
            # Create ZipInfo instance to store file information
            x = zipfile.ZipInfo(filename)
            x.extra = fp.read(centdir[zipfile._CD_EXTRA_FIELD_LENGTH])
            x.comment = fp.read(centdir[zipfile._CD_COMMENT_LENGTH])
            x.header_offset = centdir[zipfile._CD_LOCAL_HEADER_OFFSET]
            (x.create_version, x.create_system, x.extract_version, x.reserved,
                x.flag_bits, x.compress_type, t, d,
                x.CRC, x.compress_size, x.file_size) = centdir[1:12]
            x.volume, x.internal_attr, x.external_attr = centdir[15:18]
            # Convert date/time code to (year, month, day, hour, min, sec)
            x._raw_time = t
            x.date_time = ( (d>>9)+1980, (d>>5)&0xF, d&0x1F,
                                     t>>11, (t>>5)&0x3F, (t&0x1F) * 2 )

            if len(x.extra) >= 4: ## This is the added check.
                x._decodeExtra()
            x.header_offset = x.header_offset + concat
            x.filename = x._decodeFilename()
            self.filelist.append(x)
            self.NameToInfo[x.filename] = x

            # update total bytes read from central directory
            total = (total + zipfile.sizeCentralDir + centdir[zipfile._CD_FILENAME_LENGTH]
                     + centdir[zipfile._CD_EXTRA_FIELD_LENGTH]
                     + centdir[zipfile._CD_COMMENT_LENGTH])

            if self.debug > 2:
                print("total %s" % total)

class ZipFileWrapper (ArchiveWrapper):

    def open_archive(self, path_to_archive):
        if not zipfile.is_zipfile(path_to_archive):
            not_found('Not a zip file')
        try:
            return NearZipFile(path_to_archive, 'r')
        except zipfile.error:
            not_found("Can't open zip file")

    def get_names(self):
        names = self.get_archive().namelist()
        directories = set()
        for name in names:
            parts = name.split('/')
            for j in range(len(parts)):
                directories.add('/'.join(parts[:j]) + '/')
        for d in directories:
            if d not in names:
                names.append(d)
        return names

    def has_member(self, member_name):
        try:
            return bool(self.get_archive().getinfo(member_name))
        except KeyError:
            return False

    def get_member_file(self, member_name):
        return BytesIO(self.get_member_response(member_name))

    def get_member_response(self, member_name):
        return self.get_archive().read(member_name)

    def get_member(self, member_name):
        return self.get_archive().read(member_name)



from datetime import datetime
from datetime import timedelta
from durus.btree import BTree, BNode512
from durus.persistent import PersistentObject
from durus.persistent_list import PersistentList
from glob import glob
from os.path import basename
from qp.lib.spec import Specified, spec, specify, add_getters, datetime_with_tz, sequence, mapping, nspec, init
from qp.lib.stored_file import StoredFile, new_file
from qp.lib.tz import UTC
from qp.lib.util import integer
from qp.pub.common import get_publisher
from qp.pub.user import User
from os import stat
from socket import gethostname


try:
    import GeoIP
    country_code_by_addr = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE).country_code_by_addr
except ImportError:
    def country_code_by_addr(x):
        return None

def get_log_db():
    return get_publisher().get_connection().get_root().get('log_db')


class LogLine (Specified): # Not persistent, but slotted

    time_is = datetime_with_tz
    status_is = int
    bytes_is = int
    generation_time_is = float
    remote_address_is = str
    country_code_is = (None, str)
    user_is = User
    method_is = str
    referrer_is = str
    agent_is = str
    path_is = str

    def __init__(self, line):
        parts = line.split()
        self.time = None
        if len(parts) == 16:
            date_parts = parts[0].split('-')
            time_parts = parts[1].split(':')
            null_user = get_publisher().get_users()['']
            if len(date_parts) == 3 and len(time_parts)==3:
                year, month, day = map(integer, date_parts)
                hour, minute, second = map(integer, time_parts)
                time = datetime(year=year, month=month, day=day, hour=hour, minute=minute, second=second, tzinfo=UTC)
                try:
                    if ':' in parts[6]:
                        try:
                            minutes, seconds = parts[6].split(':')
                            generation_time = 60 * integer(minutes) + float(seconds)
                        except ValueError:
                            generation_time = 0.0
                    else:
                        generation_time = float(parts[6])

                    specify(self,
                        time=time,
                        status=integer(parts[2]),
                        bytes=integer(parts[3]),
                        generation_time=generation_time,
                        remote_address=parts[7],
                        country_code=country_code_by_addr(parts[7]),
                        user=get_publisher().get_users().get(parts[8], null_user),
                        method=parts[12],
                        path=parts[13],
                        referrer=parts[14],
                        agent=parts[-1])
                except:
                    print(parts)
                    raise

    def __eq__(self, other):
        for n in self.__slots__:
            if getattr(self, n) != getattr(other, n):
                return False
        return True

    def __str__(self):
        return ' '.join("%s=%s" % (n, getattr(self, n)) for n in self.__slots__)

    def __nonzero__(self):
        return self.time is not None

add_getters(LogLine)


class Log (PersistentObject, Specified):

    host_is = spec(str, "Host on which log was made")

    rotation_is = nspec(datetime_with_tz, "Time of rotation")

    file_is = nspec(StoredFile, "The file")

    data_is = spec(
        sequence(element_spec=LogLine, container_spec=PersistentList),
        "Parsed data from file")

    offset_is = nspec(
        int,
        "Position at end of last read log line in the file")

    def __init__(self, path):
        logfile = get_publisher().get_site().get_logfile()
        if path != logfile:
            self.rotation = datetime.strptime(path[len(logfile) + 1:], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=UTC)
            self.file = new_file(open(path))
            filename = basename(path)
            self.file.set_filename(filename)
        self.host = gethostname()
        init(self)
        self.data = PersistentList(self.parse(path))

    def parse(self, path=None):
        if path is None:
            path = get_publisher().get_site().get_logfile()
        log = open(path, 'r')
        log_size = stat(path).st_size
        if self.get_offset() is not None and log_size >= self.get_offset():
            log.seek(self.get_offset())
        else:
            self.offset = 0
        data = []
        while 1:
            line = log.readline()
            if not line:
                break
            log_line = LogLine(line)
            if log_line:
                data.append(log_line)
                self.offset = log.tell()
        return data

add_getters(Log)


class LogDatabase (PersistentObject, Specified):

    logs_is = spec(
        mapping({datetime_with_tz:Log}, BTree),
        "Logs by rotation date")

    log_lines_is = spec(
        mapping({datetime_with_tz:LogLine}, BTree),
        "Log lines rotation time")

    current_log_is = nspec(
        Log,
        "The current log.")

    def __init__(self):
        self.logs = BTree(node_constructor=BNode512)
        self.log_lines = BTree(node_constructor=BNode512)
        self.current_log = None

    def update_log_lines(self, log_lines_list):
        for log_line in log_lines_list:
            time = log_line.get_time()
            while time in self.log_lines:
                if log_line != self.log_lines.get(time):
                    time += timedelta(seconds=1e-6)
            if time not in self.log_lines:
                self.log_lines[time] = log_line

    def update(self):
        logfile = get_publisher().get_site().get_logfile()
        for path in glob(logfile + '.*'):
            try:
                rotation_time = datetime.strptime(path[len(logfile) + 1:], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=UTC)
            except ValueError:
                continue
            if rotation_time not in self.logs:
                log = Log(path)
                self.logs[rotation_time] = log
                self.update_log_lines(log.parse(path=path))
        if self.get_current_log() is None:
            self.current_log = Log(logfile)
        new_lines = self.current_log.parse()
        self.current_log.get_data().extend(new_lines)
        self.update_log_lines(new_lines)

    def get_last_log_line(self):
        return self.log_lines.get_max_item()[1]

    def gen_log_lines_in_range(self, start, end):
        for log_line in self.log_lines.items_range(start, end):
            yield log_line

    def get_log_lines(self):
        return self.log_lines

add_getters(LogDatabase)



'''
closed/durusmail/lib/mailman.py
'''

# If you have a port install of mailman, add this sym-link:
# - cd /opt/local/libexec/mailman; ln -s /opt/local/share/mailman/bin bin

from subprocess import Popen, PIPE
import sys

def cmd(args, input=None):
    try:
        p = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE).communicate(input=input)
    except OSError:
        print('subprocess popen error executing command:')
        print(' '.join(args))
        print(sys.exc_info()[1])
    return p

def is_subscribed(user, list_name):
    if not user.get_email():
        return False
    args = ['/www/mailman/bin/find_member', '-l', list_name, user.get_email()]
    stdout_output, stderr_output = cmd(args)
    if stderr_output:
        print('mailman.is_subscribed error: %s' % stderr_output)
        return None
    print stdout_output
    return list_name in stdout_output

def subscribe(user, list_name):
    if not user.get_email():
        return False, '%s has no email address' % user.get_id()
    if is_subscribed(user, list_name):
        return False, '%s is already a member of %s' % (user.get_email(), list_name)
    email = user.get_email()
    if user.format_realname():
        email = '"%s" <%s>' % (user.format_realname(), email)
    args = ['/www/mailman/bin/add_members', '-r', '-', list_name]
    stdout_output, stderr_output = cmd(args, input=email)
    if stderr_output:
        return False, stderr_output

    return True, '%s now subscribed to %s' % (user.get_email(), list_name)

def un_subscribe(user, list_name):
    if not user.get_email():
        return False, '%s has no email address' % user.get_id()
    if not is_subscribed(user, list_name):
        return False, '%s is not a member of %s' % (user.get_email(), list_name)
    args = ['/www/mailman/bin/remove_members', '-f', '-', list_name]
    stdout_output, stderr_output = cmd(args, input=user.get_email())
    if stderr_output:
        return False, stderr_output

    return True, '%s Unsubscribed from %s' % (user.get_email(), list_name)

def list_lists():
    args = ['/www/mailman/bin/list_lists', '-a']
    stdout_output, stderr_output = cmd(args)
    if stderr_output:
        print('mailman.list_lists error: %s' % stderr_output)
        return None
    lists = []
    for line in stdout_output.split('\n'):
        if ' - ' in line:
            list_realname, list_description = line.split(' - ', 1)
            lists.append((list_realname.strip(), list_description))
    return lists

def get_list_realname_and_description(list_name):
    for list_realname, list_description in list_lists() or []:
        if list_realname.lower() == list_name:
            return list_realname, list_description



"""
StoredFile has moved to qp.lib.stored_file.
This module should be removed after all existing StoredFiles are repickled
using the correct import.
"""
from qp.lib.stored_file import StoredFile

StoredFile # quiet checker
print("Please repickle all StoredFile instances and references.")




"""
open/dulcinea/lib/session.py
"""
from dulcinea.attachable import Attachable
from dulcinea.user import DulcineaUser
from qp.lib.spec import Specified
from qp.pub.session import Session


class DulcineaSession (Session, Specified, Attachable):

    effective_user_is = DulcineaUser
    owner_is = DulcineaUser

    def __init__(self):
        Session.__init__(self)
        Attachable.__init__(self)

    def clear_app_state(self):
        """Override to clear any session data
        """
    def set_items_per_page(self, value):
        """Override if you want this to really save something.
        """

    def get_items_per_page(self):
        return 10



"""
open/dulcinea/lib/item.py
"""
from dulcinea.base import DulcineaPersistent
from dulcinea.common import format_date
from dulcinea.partition import PartKeyed, Partition
from dulcinea.timestamped import Timestamped, reverse_timestamp_sorted
from durus.btree import BTree
from qp.lib.spec import add_getters_and_setters, string, boolean, init


class Item(DulcineaPersistent, PartKeyed, Timestamped):

    title_is = (string, None)
    approved_is = boolean

    def __init__(self):
        assert self.__class__ is not Item, "abstract class"
        PartKeyed.__init__(self)
        Timestamped.__init__(self)
        init(self, approved=False)

    def is_new(self):
        return self.key is None

    def is_expired(self):
        if self.part_key == 'expired':
            return True
        return False

    def absorb(self, other):
        assert self.__class__ is other.__class__
        for key, value in other.__getstate__().items():
            if key != 'key':
                setattr(self, key, value)

    def get_search_text(self):
        return ' '.join(self.get_search_text_fields())

    def get_search_text_fields(self):
        return [self.get_title() or '', format_date(self.get_timestamp())]

    def is_approved(self):
        return self.approved

    def can_modify(self, user):
        return user.is_admin()

add_getters_and_setters(Item)


class ItemFolder(Partition):

    def __init__(self, value_spec=Item):
        Partition.__init__(self, value_spec=value_spec, part_key_spec=('active', 'expired'))
        init(self)
        self.parts['active'] = BTree()
        self.parts['expired'] = BTree()

    def get_all(self):
        return self.get_active() + self.get_expired()

    def get_active(self):
        return list(self.get_part('active').itervalues())

    def get_expired(self):
        return list(self.get_part('expired').itervalues())

    def get_recent_items(self, count=None, include_unapproved=False):
        """(count : int = None, include_unapproved : bool = False) -> [Item]

        Return the 'count' most recent items.  If 'include_unapproved' is True,
        both un-approved and approved items are returned.
        The list returned is in reverse-chronological order by timestamp.
        """
        return reverse_timestamp_sorted(
            [item for item in self.get_active()
             if item.is_approved() or include_unapproved])[:count]

    def get_to_oldest_date(self, oldest_date, **kwargs):
        if oldest_date is not None:
            return [item for item in self.get_recent_items(**kwargs)
                    if item.get_timestamp() >= oldest_date]
        return self.get_recent_items(**kwargs)


def partition_item_folder(item_folder, item_class):
    '''Convert a Keep-style ItemFolder and Items within into a two part Partition
    '''
    from durus.persistent import PersistentObject
    from durus.btree import BTree

    def __setstate__(self, state):
        if 'mapping' in state:
            parts = BTree()
            parts['active'] = state['mapping']
            parts['expired'] = BTree()
            state['parts'] = parts
            del state['mapping']
        PersistentObject.__setstate__(self, state)

    item_folder.__class__.__setstate__ = __setstate__
    item_folder.value_spec = item_class
    item_folder.part_key_spec = ('active', 'expired')
    item_folder._p_note_change()

    for item in item_folder.get_part('active').itervalues():
        item.part_key = 'active'



"""
open/dulcinea/lib/test/utest_item.py
"""
from datetime import datetime
from dulcinea.item import Item, ItemFolder
from qp.lib.tz import Eastern
from sancho.utest import UTest

class MyItem(Item):
    pass

class TestItem (UTest):
    def _pre(self):
        self.item1 = MyItem()
        self.item1.timestamp = datetime(2003,1,1,tzinfo=Eastern)
        self.item1.set_approved(True)
        self.item1a = MyItem()
        self.item1a.timestamp = datetime(2003,1,1,tzinfo=Eastern)
        self.item2 = MyItem()
        self.item2.timestamp = datetime(2005,1,1,tzinfo=Eastern)
        self.item2.set_approved(True)
        self.item3 = MyItem()
        self.item3.timestamp = datetime(2006,1,1,tzinfo=Eastern)
        self.item3.set_approved(True)
        self.folder = ItemFolder()

    def check_basic(self):
        assert self.item1.is_new()
        self.folder.add(self.item1, 'active')
        assert not self.item1.is_new()
        self.folder.add(self.item2, 'active')
        assert set(self.folder.get_active()) == set([self.item1, self.item2])
        assert self.folder.get_recent_items(count=2) == [self.item2, self.item1]
        self.folder.add(self.item3, 'active')
        assert self.folder.get_recent_items(count=2) == [self.item3, self.item2]
        assert self.folder.get_recent_items(count=3) == [
            self.item3, self.item2, self.item1]
        assert self.folder.get_recent_items() == [
            self.item3, self.item2, self.item1]
        self.folder.add(self.item1a, 'active')
        assert self.folder.get_recent_items(count=2) == [
            self.item3, self.item2]
        self.item3.absorb(self.item2)
        assert self.item3.timestamp == self.item2.timestamp

    def check_partition(self):
        assert not self.folder.get_part('active')
        assert not self.folder.get_part('expired')
        assert self.item1.is_new()
        self.folder.add(self.item1, 'active')
        assert self.folder.get_active() == [self.item1]
        self.folder.add(self.item2, 'active')
        assert self.folder.get_active() == [self.item1, self.item2]
        assert self.folder.get_expired() == []
        self.folder.move(self.item1, 'expired')
        assert self.folder.get_active() == [self.item2]
        assert self.folder.get_expired() == [self.item1]
        assert self.folder.get(self.item1.key) is self.item1
        assert self.folder.get(self.item2.key) is self.item2
        self.folder.remove(self.item1)
        assert self.folder.get_expired() == []
        assert self.folder.get_active() == [self.item2]

    def check_delete(self):
        self.folder.add(self.item1, 'active')
        self.folder.add(self.item1a, 'active')
        item = MyItem() # auto timestamp
        self.folder.add(item, 'active')
        assert set(self.folder.get_active()) == set(
            [self.item1, self.item1a, item])
        self.folder.remove(self.item1a)
        assert set(self.folder.get_active()) == set([self.item1, item])

if __name__ == "__main__":
    TestItem()



#!/usr/bin/env python
"""
open/dulcinea/setup.py

Setup script for Dulcinea
"""
Dulcinea_Version = "0.22"

from distutils import core
from distutils.command.build_py import build_py
from distutils.command.sdist import sdist
from glob import glob
import re, sys, os

class DurusWorks_build_py (build_py):

    def find_package_modules(self, package, package_dir):
        self.check_package(package, package_dir)
        module_files = (glob(os.path.join(package_dir, "*.py")) +
                        glob(os.path.join(package_dir, "*.qpy")))
        modules = []
        setup_script = os.path.abspath(self.distribution.script_name)
        for f in module_files:
            abs_f = os.path.abspath(f)
            module = os.path.splitext(os.path.basename(f))[0]
            modules.append((package, module, f))
        return modules

    def build_module(self, module, module_file, package):
        if type(package) is str:
            package = package.split('.')
        elif type(package) not in (list, tuple):
            raise TypeError(
                "'package' must be a string (dot-separated), list, or tuple")
        # Now put the module source file into the "build" area.
        outfile = self.get_module_outfile(self.build_lib, package, module)
        if module_file.endswith(".qpy"):
            outfile = outfile[0:outfile.rfind('.')] + ".qpy"
        dir = os.path.dirname(outfile)
        self.mkpath(dir)
        return self.copy_file(module_file, outfile, preserve_mode=0)

    def byte_compile(self, files):
        try:
            from qpy.compile import compile_qpy_file
        except ImportError:
            build_py.byte_compile(self, files)
        else:
            existing = []
            for file in files:
                if os.path.exists(file):
                    existing.append(file)
                else:
                    alt = file[:-3] + ".qpy"
                    compile_qpy_file(alt)
                    sys.stdout.write('byte-compiling %s\n' % alt)
            build_py.byte_compile(self, existing)

class DurusWorks_sdist (sdist):

    def get_file_list(self):
        sdist.get_file_list(self)
        self.filelist.files = []
        self.filelist.findall()
        print('')
        for name in sorted(self.filelist.allfiles):
            if ".svn" in name:
                continue
            if name.endswith('.pyc'):
                continue
            if name.endswith('.o'):
                continue
            if name.endswith('.so'):
                continue
            if name.endswith('.class'):
                continue
            if name.endswith('~'):
                continue
            if name.endswith('#'):
                continue
            if name.startswith('build/'):
                continue
            if name.startswith('dist/'):
                continue
            self.filelist.files.append(name)
            print('ADDING', name)
        print

    def write_manifest(self):
        pass

if 'sdist' in sys.argv:
    if sys.platform == 'darwin':
        # Omit extended attributes from tarfile
        os.environ['COPYFILE_DISABLE'] = 'true'
    # Make sure that version numbers have all been updated.
    PAT = re.compile(r'\b%s\b' % re.escape(Dulcinea_Version))
    assert len(PAT.findall(open("LICENSE.txt").read())) == 12, Dulcinea_Version
    assert PAT.search(open("CHANGES.txt").readline()), Dulcinea_Version
    assert len(PAT.findall(open("README.txt").read())) == 1, Dulcinea_Version

    # Make sure that copyright statements are current.
    from datetime import datetime
    year = datetime.now().year # no tz, ok
    copyright = "Copyright (c) Corporation for National Research Initiatives %s" % year
    assert open("README.txt").read().count(copyright) == 1

kw = dict(
    name="Dulcinea",
    version=Dulcinea_Version,
    description =("A package of modules useful for developing "
                      "DurusWorks applications."),
    author="CNRI",
    author_email="webmaster@mems-exchange.org",
    url="http://www.mems-exchange.org/software/dulcinea/",
    license="see LICENSE.txt",
    package_dir={'dulcinea':'lib'},
    packages=['dulcinea',
        'dulcinea.property',
        'dulcinea.ui',
        'dulcinea.ui.lib',
        'dulcinea.ui.form',
        'dulcinea.ui.user'],
    scripts=['bin/expire_session.py'],
    cmdclass=dict(
        build_py=DurusWorks_build_py,
        sdist=DurusWorks_sdist)
    )

if hasattr(core, 'setup_keywords'):
    if 'classifiers' in core.setup_keywords:
        kw['classifiers'] = ['Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Operating System :: Unix',
          'Topic :: Software Development :: Libraries',
          ]
    if 'download_url' in core.setup_keywords:
        kw['download_url'] = ('http://www.mems-exchange.org/software/files'
                              '/dulcinea/Dulcinea-%s.tar.gz' % kw['version'])

core.setup(**kw)
